/*
 * libslb.h - internal header file.
 * Used only when compiling the shared library code,
 * to redirect standard library functions to the
 * library that was linked to the application
 *
 * Copyright (C) 2019 Thorsten Otto
 */

#ifndef __TIFFSLB_H__
#define __TIFFSLB_H__ 1

#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <setjmp.h>
#include <unistd.h>

#undef errno
#undef __set_errno
#undef stderr
#undef strerror

#undef memset
#undef memcpy
#undef memcmp
#undef memmove
#undef memchr

#undef strlen
#undef strcpy
#undef strcat
#undef strcmp
#undef strncmp

#undef malloc
#undef calloc
#undef realloc
#undef free

#undef open
#undef close
#undef read
#undef write
#undef lseek

#undef fwrite
#undef sprintf
#undef snprintf
#undef vsnprintf
#undef fprintf
#undef vfprintf
#undef fputc

#undef sigsetjmp
#undef setjmp
#undef longjmp

#undef rand
#undef srand
#undef getenv

#include "libstrct.h"

#define errno (tiff_get_slb_funcs()->p_get_errno())
#define __set_errno(e) (tiff_get_slb_funcs()->p_set_errno(e))
#define stderr (tiff_get_slb_funcs()->stderr_location)
#define strerror tiff_get_slb_funcs()->p_strerror

#define memset(d, c, l) tiff_get_slb_funcs()->p_memset(d, c, l)
#define memcpy(d, s, l) tiff_get_slb_funcs()->p_memcpy(d, s, l)
#define memmove(d, s, l) tiff_get_slb_funcs()->p_memcpy(d, s, l)
#define memcmp(d, s, l) tiff_get_slb_funcs()->p_memcmp(d, s, l)
#define memchr(d, c, n) tiff_get_slb_funcs()->p_memchr(d, c, n)

#define strlen tiff_get_slb_funcs()->p_strlen
#define strcpy tiff_get_slb_funcs()->p_strcpy
#define strcat tiff_get_slb_funcs()->p_strcat
#define strcmp tiff_get_slb_funcs()->p_strcmp
#define strncmp tiff_get_slb_funcs()->p_strncmp

#define malloc tiff_get_slb_funcs()->p_malloc
#define calloc tiff_get_slb_funcs()->p_calloc
#define realloc tiff_get_slb_funcs()->p_realloc
#define free(p) tiff_get_slb_funcs()->p_free(p)

#define open tiff_get_slb_funcs()->p_open
#define close tiff_get_slb_funcs()->p_close
#define read tiff_get_slb_funcs()->p_read
#define write tiff_get_slb_funcs()->p_write
#define lseek tiff_get_slb_funcs()->p_lseek

#define fwrite tiff_get_slb_funcs()->p_fwrite
#define sprintf tiff_get_slb_funcs()->p_sprintf
#define snprintf tiff_get_slb_funcs()->p_snprintf
#define vsnprintf tiff_get_slb_funcs()->p_vsnprintf
#define fprintf tiff_get_slb_funcs()->p_fprintf
#define vfprintf tiff_get_slb_funcs()->p_vfprintf
#define fputc tiff_get_slb_funcs()->p_fputc

#define sigsetjmp tiff_get_slb_funcs()->p_sigsetjmp
#define setjmp(j) tiff_get_slb_funcs()->p_sigsetjmp(j, 1)
#define longjmp tiff_get_slb_funcs()->p_longjmp

#define rand tiff_get_slb_funcs()->p_rand
#define srand tiff_get_slb_funcs()->p_srand
#define getenv tiff_get_slb_funcs()->p_getenv

/*
 * used to load libraries only if needed
 */
long slb_tiff_zlib_open(void);
long slb_tiff_lzma_open(void);
long slb_tiff_jpeg_open(void);
long slb_tiff_zstd_open(void);
long slb_tiff_webp_open(void);
long slb_tiff_jbig_open(void);

#endif /* __TIFFSLB_H__ */
