/*
 * pcmath8.c - float conversion functions for Pure-C
 * (m6888x version)
 *
 * Copyright (C) 2018 Thorsten Otto
 */

#include "tiffio.h"
#include "pcmath.h"

#if !defined(__HAVE_68881__)
# error "this must be compiled for 68881"
#endif

union ieeefloat {
	__gcc_float_t v;
	float f;
};

union ieeedouble {
	struct {
		unsigned short sign_exp;
		unsigned short empty;
		unsigned long hi;
		unsigned long lo;
	} p;
	double d;
};


struct __gcc_double pc_d96tod64(double x)
{
	union ieeedouble y;
	struct __gcc_double g;
	unsigned short exp;
	unsigned long hi;
	
	y.d = x;
	exp = y.p.sign_exp & FP_EXTENDED_EXP_MAX;
	hi = y.p.hi & 0x7fffffffUL;
	if (exp == 0)
	{
		g.hi = g.lo = 0;
	} else if (exp == FP_EXTENDED_EXP_MAX)
	{
		g.hi = (unsigned long)FP_DOUBLE_EXP_MAX << 20;
		if (hi != 0 || y.p.lo != 0)
		{
			g.hi |= 0xfffffUL;
			g.lo = 0xffffffffUL;
		} else
		{
			g.lo = 0;
		}
	} else if (exp >= (FP_EXTENDED_EXP_BIAS + FP_DOUBLE_EXP_MAX - FP_DOUBLE_EXP_BIAS))
	{
		g.hi = (unsigned long)FP_DOUBLE_EXP_MAX << 20;
		g.lo = 0;
	} else if (exp < (FP_EXTENDED_EXP_MAX - FP_EXTENDED_EXP_BIAS - FP_DOUBLE_EXP_BIAS))
	{
		g.hi = g.lo = 0;
	} else
	{
		g.hi = (unsigned long)(exp - FP_EXTENDED_EXP_BIAS + FP_DOUBLE_EXP_BIAS) << 20;
		g.hi |= hi >> 11;
		g.lo = (hi << 21) | (y.p.lo >> 11);
	}
	if (y.p.sign_exp & 0x8000)
		g.hi |= 0x80000000UL;
	return g;
}


double pc_d64tod96(struct __gcc_double x)
{
	union ieeedouble y;
	unsigned short exp;
	unsigned long hi;

	exp = (unsigned short)(x.hi >> 20) & FP_DOUBLE_EXP_MAX;
	hi = x.hi & 0xfffffUL;
	if (exp == 0)
	{
		y.p.sign_exp = 0;
		y.p.hi = y.p.lo = 0;
	} else if (exp == FP_DOUBLE_EXP_MAX)
	{
		y.p.sign_exp = FP_EXTENDED_EXP_MAX;
		if (hi != 0 || x.lo != 0)
		{
			y.p.hi = y.p.lo = 0xffffffffUL;
		} else
		{
			y.p.hi = y.p.lo = 0;
		}
	} else
	{
		y.p.sign_exp = (exp - FP_DOUBLE_EXP_BIAS) + FP_EXTENDED_EXP_BIAS;
		y.p.hi = (hi << 11) | 0x80000000UL | (x.lo >> 21);
		y.p.lo = x.lo << 11;
	}
	if (x.hi & 0x80000000UL)
		y.p.sign_exp |= 0x8000U;
	y.p.empty = 0;
	return y.d;
}


float pc_float_from_gcc8(__gcc_float_t x)
{
	union ieeefloat v;
	v.v = x;
	return v.f;
}

__gcc_float_t pc_float_to_gcc8(float x)
{
	union ieeefloat v;
	v.f = x;
	return v.v;
}
